import { mergeTypeDefs } from "./typedefs-mergers/merge-typedefs";
import { asArray } from "../utils/helpers";
import { mergeResolvers } from "./resolvers-mergers/merge-resolvers";
import { extractResolversFromSchema, composeResolvers, buildSchemaWithResolvers } from "../utils";
export function mergeSchemas({ schemas, typeDefs, resolvers, resolversComposition, exclusions, }) {
    return buildSchemaWithResolvers({
        typeDefs: mergeTypeDefs([
            ...schemas,
            ...typeDefs ? asArray(typeDefs) : []
        ], { exclusions }),
        resolvers: composeResolvers(mergeResolvers([
            ...schemas.map(schema => extractResolversFromSchema(schema)),
            ...resolvers ? asArray(resolvers) : []
        ], { exclusions }), resolversComposition || {})
    });
}
export async function mergeSchemasAsync({ schemas, typeDefs, resolvers, resolversComposition, exclusions, }) {
    const [typeDefsOutput, resolversOutput,] = await Promise.all([
        mergeTypeDefs([
            ...schemas,
            ...typeDefs ? asArray(typeDefs) : []
        ], { exclusions }),
        Promise
            .all(schemas.map(async (schema) => extractResolversFromSchema(schema)))
            .then(extractedResolvers => composeResolvers(mergeResolvers([
            ...extractedResolvers,
            ...resolvers ? asArray(resolvers) : []
        ], { exclusions }), resolversComposition || {})),
    ]);
    return buildSchemaWithResolvers({
        typeDefs: typeDefsOutput,
        resolvers: resolversOutput
    });
}
//# sourceMappingURL=merge-schemas.js.map